from paytmpg.pg.utils.stringUtil import make_string, equals


class NativePaymentStatusRequestBody:

    def __init__(self):
        self.mid = None
        self.orderId = None
        self.txnType = None
        self.fromAoaMerchant = None

    def set_mid(self, mid):
        self.mid = mid

    def get_mid(self):
        return self.mid

    def set_order_id(self, order_id):
        self.orderId = order_id

    def get_order_id(self):
        return self.orderId

    def set_txn_type(self, txn_type):
        self.txnType = txn_type

    def get_txn_type(self):
        return self.txnType

    def set_from_aoa_merchant(self, from_aoa_merchant):
        self.fromAoaMerchant = from_aoa_merchant

    def get_from_aoa_merchant(self):
        return self.fromAoaMerchant

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)



from paytmpg.merchant.models.Time import current_time_millis
from paytmpg.pg.constants.LibraryConstants import LibraryConstants
from paytmpg.pg.utils.stringUtil import make_string, equals


class ResponseHeader(object):

    def __init__(self):
        self.responseTimestamp = current_time_millis()
        self.version = LibraryConstants.VERSION

    def get_response_time_stamp(self):
        return self.responseTimestamp

    def set_response_time_stamp(self, response_time_stamp):
        self.responseTimestamp = response_time_stamp

    def get_version(self):
        return self.version

    def set_version(self, version):
        self.version = version

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)



from sys import exc_info

from paytmpg.pg.constants.MerchantProperty import MerchantProperty

from paytmpg.pg.utils.SignatureUtil import generateSignature
from paytmpg.pg.utils.CommonUtil import CommonUtil
from paytmpg.pg.utils.stringUtil import is_empty, format_string

from paytmpg.pg.request.RefundInitiateRequest import RefundInitiateRequest
from paytmpg.pg.response.AsyncRefundResponse import AsyncRefundResponse
from paytmpg.pg.response.NativeRefundStatusResponse import NativeRefundStatusResponse
from paytmpg.pg.request.NativeRefundStatusRequest import NativeRefundStatusRequest
from paytmpg.pg.Request import Request
from paytmpg.pg.SDKException import SDKException
from paytmpg.merchant.models.SDKResponse import SDKResponse


class Refund:
    """This class is handle all the Refund calls from DemoApp.py and create
    request objects and make call to the respective controller
    This class receive the Paytm response objects and translate the to their
    respective merchant response objects and returns call to DemoApp
    """
    @classmethod
    def doRefund(cls, refund_detail):
        """
        :param refund_detail: all parameter are in refund detail which is required to make request
        :return: SDKResponse object
        """
        MerchantProperty.set_logger_name(CommonUtil.get_unique_request_id())
        try:
            if not MerchantProperty.is_initialized:
                MerchantProperty.logger.debug("Refund :: do_refund :: MerchantProperties are not initialized.")
                return CommonUtil.get_sdk_response(SDKException.get_merchant_property_initialization_exception(),
                                                   AsyncRefundResponse())
            MerchantProperty.logger.info("Refund :: do_refund for refund_detail: {}".format(refund_detail))
            cls.__validate_refund(refund_detail)
            request = cls.__create_refund_initiate_request(refund_detail)
            cls.__set_signature(request.get_head(), request.get_body())
            response = SDKResponse().set_response_object(AsyncRefundResponse())
            Request.process(request, MerchantProperty.get_refund_url(), response,
                            refund_detail.get_read_timeout())
            return response
        except Exception:
            return CommonUtil.get_sdk_response(exc_info()[1], AsyncRefundResponse())

    @staticmethod
    def __set_signature(head, body):
        """
        :param head: in which checksum attribute will be added
        :param body: value of checksum will be created over it's string value
        :return: None. As head is augmented by signature value
        """
        MerchantProperty.logger.debug("Refund:: set signature :: body is {}".format(format_string(body)))
        signature = generateSignature(format_string(body), MerchantProperty.get_merchant_key())
        head.set_signature(signature)

    @staticmethod
    def __create_refund_initiate_request(refund_detail):
        """
        :param refund_detail: has all parameter
        :return: RefundInitiateRequest object
        """
        head = CommonUtil.get_secure_request_header(MerchantProperty.get_client_id())
        body = refund_detail._RefundDetail__create_refund_initiate_request_body()
        return RefundInitiateRequest().set_head(head).set_body(body)

    @staticmethod
    def __validate_refund(refund_detail):
        """validate_refund checks if all mandatory parameters are present for Refund api
        If not, then is will throw the RequestValidationException exception
        :param refund_detail: RefundInitiateRequest object
        :return: raise exception if any mandatory parameter is missing
        """
        MerchantProperty.logger.info("Refund :: validate_refund for refund_detail : {}".format(refund_detail))
        if is_empty(refund_detail.get_order_id())\
                or is_empty(refund_detail.get_ref_id())\
                or is_empty(refund_detail.get_txn_id())\
                or is_empty(refund_detail.get_refund_amount()):
            raise SDKException.get_missing_mandatory_parameters_exception()

    @classmethod
    def getRefundStatus(cls, refund_status_detail):
        MerchantProperty.set_logger_name(CommonUtil.get_unique_request_id())
        try:
            if not MerchantProperty.is_initialized:
                MerchantProperty.logger.debug("Refund :: get_refund_status :: MerchantProperties are not initialized.")
                return CommonUtil.get_sdk_response(SDKException.get_merchant_property_initialization_exception(),
                                                   NativeRefundStatusResponse())
            MerchantProperty.logger.info("Refund :: get_refund_status : for refund status detail: {}".format(refund_status_detail))
            cls.__validate_refund_status(refund_status_detail)
            request = cls.__create_refund_status_request(refund_status_detail)
            cls.__set_signature(request.get_head(), request.get_body())
            response = SDKResponse().set_response_object(NativeRefundStatusResponse())
            Request.process(request, MerchantProperty.get_refund_status_url(), response, refund_status_detail.get_read_timeout())
            return response
        except Exception:
            return CommonUtil.get_sdk_response(exc_info()[1], NativeRefundStatusResponse())

    @staticmethod
    def __create_refund_status_request(refund_status_detail):
        """
        :param refund_status_detail:
        :return: NativeRefundStatusRequest object
        """
        head = CommonUtil.get_secure_request_header(MerchantProperty.get_client_id(), None)
        body = refund_status_detail._RefundStatusDetail__create_native_refund_status_request_body()
        return NativeRefundStatusRequest().set_head(head).set_body(body)

    @staticmethod
    def __validate_refund_status(refund_status_detail):
        MerchantProperty.logger.info("Refund :: validate refund status for refund_status_detail: {}".format(refund_status_detail))
        if is_empty(refund_status_detail.get_order_id())\
                or is_empty(refund_status_detail.get_ref_id()):
            MerchantProperty.logger.debug("validate refund status is false")
            raise SDKException.get_missing_mandatory_parameters_exception()
        MerchantProperty.logger.debug("validate refund_status true")

    def __dir__(self):
        return ""



"""This file has functionality regarding dunder function which is used many times
"""


def make_string(arg):
    """Used to overload Dunder method __str__ of any class
    This is used to convert <class 'class_name'> to string form and this is also a valid json
    :param arg: arg ~ self, iterate over it and form key-value {attribute: value}
    :return: string value of self(arg) of any class
    """
    res = str()
    is_first = True
    for k, v in arg.__dict__.items():
        if v is not None and v is not "":
            if not is_first:
                res += ','
            if v.__str__()[0] == '{' or v.__str__()[0] == '[':
                res += '"{}":{}'.format(k, v)
            else:
                res += '"{}":"{}"'.format(k, v)
            is_first = False
    if is_first:
        return None
    return '{' + res + '}'


def equals(first, second):
    """
    used to overload Dunder method __eq__ of any class
    Compare equality of first and second
    :param first: self of one object
    :param second: self of other object
    :return: True if all attributes value are same otherwise False
    """
    for k, v in first.__dict__.items():
        if not hasattr(second, k) or v != second.__dict__[k]:
            return False
    for k, v in second.__dict__.items():
        if not hasattr(first, k) or v != first.__dict__[k]:
            return False
    return True


def is_empty(obj):
    """
    check for "", "   " and None
    :param obj:data to be tested
    :return: True if it is one of "" or "  " or None otherwise False
    """
    if not obj or not obj.strip():
        return True
    else:
        return False


def format_string(string):
    string = str(string)
    return string.replace('\'', '\"').replace(' ', '')



from paytmpg.pg.response.BaseResponseBody import BaseResponseBody
from paytmpg.pg.response.ChildTransaction import ChildTransaction
from paytmpg.pg.utils.stringUtil import make_string, equals


class NativePaymentStatusResponseBody(BaseResponseBody):

    def __init__(self):
        self.customMerchantResponse = None
        self.customChecksumString = None
        self.txnId = None
        self.bankTxnId = None
        self.orderId = None
        self.txnAmount = None
        self.txnType = None
        self.gatewayName = None
        self.bankName = None
        self.mid = None
        self.paymentMode = None
        self.refundAmt = None
        self.txnDate = None
        self.refundId = None
        self.refId = None
        self.childTransaction = None
        self.subsId = None
        self.merchantUniqueReference = None
        self.blockedAmount = None
        self.preAuthId = None
        self.maskedCardNo = None
        self.cardIndexNo = None
        self.maskedCustomerMobileNumber = None
        self.posId = None
        self.uniqueReferenceLabel = None
        self.uniqueReferenceValue = None
        self.pccCode = None
        self.prn = None
        self.udf1 = None
        self.udf2 = None
        self.udf3 = None
        self.comments = None
        self.currentTxnCount = None
        self.loyaltyPoints = None
        super(NativePaymentStatusResponseBody, self).__init__()

    def set_txn_id(self, txn_id):
        self.txnId = txn_id

    def get_txn_id(self):
        return self.txnId

    def set_bank_txn_id(self, bank_txn_id):
        self.bankTxnId = bank_txn_id

    def get_bank_txn_id(self):
        return self.bankTxnId

    def set_order_id(self, order_id):
        self.orderId = order_id

    def get_order_id(self):
        return self.orderId

    def set_txn_amount(self, txn_amount):
        self.txnAmount = txn_amount

    def get_txn_amount(self):
        return self.txnAmount

    def set_txn_type(self, txn_type):
        self.txnType = txn_type

    def get_txn_type(self):
        return self.txnType

    def set_gateway_name(self, gateway_name):
        self.gatewayName = gateway_name

    def get_gateway_name(self):
        return self.gatewayName

    def set_bank_name(self, bank_name):
        self.bankName = bank_name

    def get_bank_name(self):
        return self.bankName

    def set_mid(self, mid):
        self.mid = mid

    def get_mid(self):
        return self.mid

    def set_payment_mode(self, payment_mode):
        self.paymentMode = payment_mode

    def get_payment_mode(self):
        return self.paymentMode

    def set_refund_amt(self, refund_amt):
        self.refundAmt = refund_amt

    def get_refund_amt(self):
        return self.refundAmt

    def set_txn_date(self, txn_date):
        self.txnDate = txn_date

    def get_txn_date(self):
        return self.txnDate

    def set_refund_id(self, refund_id):
        self.refundId = refund_id

    def get_refund_id(self):
        return self.refundId

    def set_ref_id(self, ref_id):
        self.refId = ref_id

    def get_ref_id(self):
        return self.refId

    def set_child_transaction(self, child_transaction):
        self.childTransaction = child_transaction

    def get_child_transaction(self):
        return self.childTransaction

    def set_subs_id(self, subs_id):
        self.subsId = subs_id

    def get_subs_id(self):
        return self.subsId

    def set_merchant_unique_reference(self, merchant_unique_reference):
        self.merchantUniqueReference = merchant_unique_reference

    def get_merchant_unique_reference(self):
        return self.merchantUniqueReference

    def set_blocked_amount(self, blocked_amount):
        self.blockedAmount = blocked_amount

    def get_blocked_amount(self):
        return self.blockedAmount

    def set_pre_auth_id(self, pre_auth_id):
        self.preAuthId = pre_auth_id

    def get_pre_auth_id(self):
        return self.preAuthId

    def set_masked_card_no(self, masked_card_no):
        self.maskedCardNo = masked_card_no

    def get_masked_card_no(self):
        return self.maskedCardNo

    def set_card_index_no(self, card_index_no):
        self.cardIndexNo = card_index_no

    def get_card_index_no(self):
        return self.cardIndexNo

    def set_masked_customer_mobile_number(self, masked_customer_mobile_number):
        self.maskedCustomerMobileNumber = masked_customer_mobile_number

    def get_masked_customer_mobile_number(self):
        return self.maskedCustomerMobileNumber

    def set_pos_id(self, pos_id):
        self.posId = pos_id

    def get_pos_id(self):
        return self.posId

    def set_unique_reference_label(self, unique_reference_label):
        self.uniqueReferenceLabel = unique_reference_label

    def get_unique_reference_label(self):
        return self.uniqueReferenceLabel

    def set_unique_reference_value(self, unique_reference_value):
        self.uniqueReferenceValue = unique_reference_value

    def get_unique_reference_value(self):
        return self.uniqueReferenceValue

    def set_pcc_code(self, pcc_code):
        self.pccCode = pcc_code

    def get_pcc_code(self):
        return self.pccCode

    def set_prn(self, prn):
        self.prn = prn

    def get_prn(self):
        return self.prn

    def set_udf1(self, udf1):
        self.udf1 = udf1

    def get_udf1(self):
        return self.udf1

    def set_udf2(self, udf2):
        self.udf2 = udf2

    def get_udf2(self):
        return self.udf2

    def set_udf3(self, udf3):
        self.udf3 = udf3

    def get_udf3(self):
        return self.udf3

    def set_comments(self, comments):
        self.comments = comments

    def get_comments(self):
        return self.comments

    def set_current_txn_count(self, current_txn_count):
        self.currentTxnCount = current_txn_count

    def get_current_txn_count(self):
        return self.currentTxnCount

    def set_loyalty_points(self, loyalty_points):
        self.loyaltyPoints = loyalty_points

    def get_loyalty_points(self):
        return self.loyaltyPoints

    def set_custom_merchant_response(self, custom_merchant_response):
        self.customMerchantResponse = custom_merchant_response

    def get_custom_merchant_response(self):
        return self.customMerchantResponse

    def set_custom_checksum_string(self, custom_checksum_string):
        self.customChecksumString = custom_checksum_string

    def get_custom_checksum_string(self):
        return self.customChecksumString

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)



from paytmpg.pg.constants.ErrorConstants import ErrorConstants
from paytmpg.pg.constants.MerchantProperty import MerchantProperty
from paytmpg.pg.utils.stringUtil import make_string


class SDKException(Exception):
    """RequestValidationException creates exception which are originated from Merchant code
    """
    raw_data = None

    def __init__(self, msg):
        self.msg = msg
        self.raw_data = None
        MerchantProperty.logger.critical("SDKException")

    def get_msg(self):
        return self.msg

    def set_raw_data(self, raw_data):
        self.raw_data = raw_data
        return self

    def get_raw_data(self):
        return self.raw_data

    def __str__(self):
        return make_string(self)

    @staticmethod
    def get_merchant_property_initialization_exception():
        """
        :return: SDKException when merchant have not initialized parameter
        """
        MerchantProperty.logger.critical("SDKException :: " + ErrorConstants.ErrorMessage.MISSING_MERCHANT_PROPERTY)
        return SDKException(ErrorConstants.ErrorMessage.MISSING_MERCHANT_PROPERTY);

    @staticmethod
    def get_missing_mandatory_parameters_exception():
        """
        :return: SDKException when any mandatory parameter is missing
        """
        MerchantProperty.logger.critical("SDKException :: " + ErrorConstants.ErrorMessage.MISSING_MANDATORY_PARAMETERS)
        return SDKException(ErrorConstants.ErrorMessage.MISSING_MANDATORY_PARAMETERS)

    @staticmethod
    def get_transaction_token_exception():
        """
        :return: SDKException when transaction token is missing
        """
        MerchantProperty.logger.critical("SDKException :: " + ErrorConstants.ErrorMessage.MISSING_TRANSACTION_TOKEN)
        return SDKException(ErrorConstants.ErrorMessage.MISSING_TRANSACTION_TOKEN)

    @staticmethod
    def get_signature_validation_failed_exception(response_received):
        """
        :param response_received: response received in api hit
        :return: SDKException when Signature validation failed at merchant side
        """
        MerchantProperty.logger.critical("SDKException :: " + ErrorConstants.ErrorMessage.SIGNATURE_VALIDATION_FAILED)
        return SDKException(ErrorConstants.ErrorMessage.SIGNATURE_VALIDATION_FAILED).set_raw_data(response_received)

    @staticmethod
    def get_sdk_exception_with_json_data(exception_message, json_object):
        """
        :param exception_message: exception message
        :param json_object: json object received in api hit
        :return: SDKException when exception occur after api hit
        """
        MerchantProperty.logger.critical("SDKException :: " + exception_message)
        return SDKException(exception_message).set_raw_data(json_object)

    @staticmethod
    def get_sdk_exception(exception_message):
        """
        :param exception_message: exception message
        :return: SDKException when exception occur before api hit
        """
        MerchantProperty.logger.critical("SDKException :: " + exception_message)
        return SDKException(exception_message)



import logging

from paytmpg.pg.constants.LibraryConstants import LibraryConstants


class MerchantProperty:
    """This class is used to store all the merchant related constants that are
     common to all payments and orders
    """
    """This is true if merchant initialized the required parameter
    which is set by calling initialize method of this class.
    """
    is_initialized = False

    environment = LibraryConstants.STAGING_ENVIRONMENT

    """timeout constants in seconds default"""
    connect_timeout = 30
    read_timeout = 80

    """providing key and merchant id"""
    mid = None
    """WEBSITE should be webstaging for testing purpose
    for production it's value defined in documents"""
    website = "WEBSTAGING"
    merchant_key = ""

    client_id = ""

    """callback url on which paytmpg will respond for api calls"""
    callback_url = "https://pg-staging.paytm.in/MerchantSite/bankResponse"

    base_url = "https://securegw-stage.paytm.in"
    initiate_txn_url = base_url + "/theia/api/v1/initiateTransaction"
    refund_url = base_url + "/refund/api/v1/async/refund"
    payment_status_url = base_url + "/merchant-status/api/v1/getPaymentStatus"
    refund_status_url = "https://pgp-ite.paytm.in/refund/api/v1/refundStatus"

    """Logging instance 
    Used for logging information according to set level which can be changed while initializing parameter of merchant
    """
    logger = logging.getLogger("Paytm")
    log_handler = logging.StreamHandler()
    formatter = logging.Formatter("%(name)s: %(levelname)s: %(message)s")
    log_handler.setFormatter(formatter)
    logger_disable = True
    logging_level = logging.DEBUG
    request_id = ""

    @classmethod
    def set_logger_name(cls, request_id):
        cls.logger.name = "Paytm " + request_id
        cls.request_id = request_id

    @classmethod
    def get_mid(cls):
        return cls.mid

    @classmethod
    def set_mid(cls, mid):
        cls.mid = mid

    @classmethod
    def get_website(cls):
        return cls.website

    @classmethod
    def set_website(cls, website):
        cls.website = website

    @classmethod
    def get_environment(cls):
        return cls.environment

    @classmethod
    def set_log_handler(cls, handler):
        if type(handler) is type(logging.StreamHandler()) or str(type(handler)) == str("<class 'logging.FileHandler'>"):
            cls.log_handler = handler

    @classmethod
    def set_logging_disable(cls, logger_disable):
        if type(logger_disable) is bool:
            cls.logger_disable = logger_disable

    @classmethod
    def set_logging_level(cls, logging_level):
        if type(logging_level) is type(cls.logging_level):
            cls.logging_level = logging_level

    @classmethod
    def initialize(cls, environment, mid, merchant_key, client_id, website):
        """
        :param environment:
        :param mid:
        :param merchant_key:
        :param client_id:
        :param website:
        :param callback_url:
        :return:
        """
        cls.is_initialized = True

        """initiate logger"""
        cls.logger.setLevel(cls.logging_level)
        cls.logger.addHandler(cls.log_handler)
        cls.logger.disabled = cls.logger_disable

        cls.set_environment(environment)
        cls.set_mid(mid)
        cls.set_merchant_key(merchant_key)
        cls.set_client_id(client_id)
        cls.set_website(website)

    @classmethod
    def set_merchant_key(cls, merchant_key):
        cls.merchant_key = merchant_key

    @classmethod
    def set_base_url(cls, base_url):
        cls.base_url = base_url

    @classmethod
    def get_base_url(cls):
        return cls.base_url

    @classmethod
    def get_connect_timeout(cls):
        return cls.connect_timeout

    @classmethod
    def set_connect_timeout(cls, connect_timeout):
        cls.connect_timeout = connect_timeout

    @classmethod
    def set_read_timeout(cls, read_timeout):
        cls.read_timeout = read_timeout

    @classmethod
    def get_read_timeout(cls):
        return cls.read_timeout

    @classmethod
    def set_timeout(cls, connect_timeout, read_timeout):
        cls.connect_timeout = connect_timeout
        cls.read_timeout = read_timeout

    @classmethod
    def set_environment(cls, environment):
        cls.environment = environment
        if cls.environment == LibraryConstants.PRODUCTION_ENVIRONMENT:
            cls.base_url = "https://securegw.paytm.in"
            cls.initiate_txn_url = cls.base_url + "/theia/api/v1/initiateTransaction"
            cls.refund_url = cls.base_url + "/refund/api/v2/async/refund"
            cls.payment_status_url = cls.base_url + "/merchant-status/api/v1/getPaymentStatus"

    @classmethod
    def set_client_id(cls, client_id):
        cls.client_id = client_id

    @classmethod
    def get_client_id(cls):
        return cls.client_id

    @classmethod
    def get_merchant_key(cls):
        return cls.merchant_key

    @classmethod
    def get_callback_url(cls):
        return cls.callback_url

    @classmethod
    def set_callback_url(cls, callback_url):
        cls.callback_url = callback_url

    @classmethod
    def get_callback_url_default(cls):
        return cls.callback_url_default

    @classmethod
    def set_callback_url_default(cls, callback_url_default):
        cls.callback_url_default = callback_url_default

    @classmethod
    def get_initiate_txn_url(cls):
        return cls.initiate_txn_url

    @classmethod
    def get_refund_url(cls):
        return cls.refund_url

    @classmethod
    def get_payment_status_url(cls):
        return cls.payment_status_url


    @classmethod
    def get_refund_status_url(cls):
        return cls.refund_status_url



class ErrorConstants:
    """This class is used to store error constants Merchant can update these
    constants according to his need
    """

    UTILITY_CLASS_EXCEPTION = "Utility class cannot be instantiated"

    def __init__(self):
        raise Exception(self.UTILITY_CLASS_EXCEPTION)

    """Result Status In case of Failure"""
    FAILURE = "failure"

    class ErrorMessage:
        """Result messages in case of failure"""
        def __init__(self):
            raise Exception(ErrorConstants.UTILITY_CLASS_EXCEPTION)

        """Result message when verify signature returns false"""
        SIGNATURE_VALIDATION_FAILED = "Signature Validation Failed"

        """Result message when verify signature returns exception"""
        VALIDATE_SIGNATURE_EXCEPTION = "Validate Signature Exception"

        """Result message when verify signature returns exception"""
        GENERATE_SIGNATURE_EXCEPTION = "Generate Signature Exception"

        """Result message when any required parameter is missing in api calling"""
        MISSING_MANDATORY_PARAMETERS = "Missing Mandatory Parameters"

        """ Result message when Transaction token is missing if required in api """
        MISSING_TRANSACTION_TOKEN = "Missing Transaction Token"

        """ Result message when Merchant Property are not initialized """
        MISSING_MERCHANT_PROPERTY = "Missing Merchant Property"

        """ Result message when String to object conversion failed """
        JSONSTRING_TO_OBJECT_CONVERSION_FAILED = "JsonString to object conversion failure"

        """ Result message when Object to string conversion failed """
        OBJECT_TO_JSONSTRING_CONVERSION_FAILED = "Object to JsonString conversion failure"

    class ErrorCode:

        def __init__(self):
            raise Exception(ErrorConstants.UTILITY_CLASS_EXCEPTION)

        """Result code in case of failure"""
        DEFAULT = "501"



from base64 import b64encode, b64decode
from string import digits, ascii_uppercase, ascii_lowercase
from random import choice
from hashlib import sha256

from Crypto.Cipher import AES
IV = "@@@@&&&&####$$$$"
BLOCK_SIZE = 16


def generateSignature(param_str, merchant_key, salt=None):
    params_string = param_str
    salt = salt if salt else generateRandomString(4)
    final_string = '%s|%s' % (params_string, salt)

    hasher = sha256(final_string.encode())
    hash_string = hasher.hexdigest()

    hash_string += salt

    return encrypt(hash_string, IV, merchant_key)


def verifySignature(param_str, merchant_key, checksum):
    # Get salt
    paytm_hash = decrypt(checksum, IV, merchant_key)
    salt = paytm_hash[-4:]
    calculated_checksum = generateSignature(param_str, merchant_key, salt=salt)
    return calculated_checksum == checksum


def generateRandomString(size=6, chars=ascii_uppercase + digits + ascii_lowercase):
    return ''.join(choice(chars) for _ in range(size))


__pad__ = lambda s: s + (BLOCK_SIZE - len(s) % BLOCK_SIZE) * chr(BLOCK_SIZE - len(s) % BLOCK_SIZE)
__unpad__ = lambda s: s[0:-ord(s[-1])]


def encrypt(to_encode, iv, key):
    # Pad
    to_encode = __pad__(to_encode)
    # Encrypt
    c = AES.new(key.encode("utf8"), AES.MODE_CBC, iv.encode("utf8"))
    to_encode = c.encrypt(bytearray(to_encode.encode("utf8")))
    # Encode
    to_encode = b64encode(to_encode)
    return to_encode.decode("UTF-8")


def decrypt(to_decode, iv, key):
    # Decode
    to_decode = b64decode(to_decode.encode("UTF8"))
    # Decrypt
    c = AES.new(key.encode("utf8"), AES.MODE_CBC, iv.encode("utf8"))
    to_decode = c.decrypt(to_decode)
    if type(to_decode) == bytes:
        # convert bytes array to str.
        to_decode = to_decode.decode()
    # remove pad
    return __unpad__(to_decode)



import json

from paytmpg.pg.constants.ErrorConstants import ErrorConstants
from paytmpg.pg.SDKException import SDKException
from paytmpg.pg.constants.MerchantProperty import MerchantProperty


class JsonToObject:
    """Used to put received response (by posting request) into corresponding object
    """

    def __init__(self, arg, obj):
        """
        :param arg: this is <class 'dict'> or str which is called by Request.py for putting these value in obj
        :param obj: Response object in which value of arg to be putted
        """
        if type(arg) is str:
            arg = json.loads(arg)
        self.convert_json(arg, obj)

    def convert_json(self, arg, obj):
        """This method iterate over arg if it is list or dict
        and set the value of corresponding attribute
        :param arg: dict form of data
        :param obj: arg is putted into obj recursively
        :return: void or raise exception if attribute is not correct
        """
        self.__dict__ = {}
        for key, value in arg.items():
            if type(value) is dict:
                value = JsonToObject(value, getattr(obj, key))
            elif hasattr(obj, key):
                    obj.__dict__[key] = value
            else:
                MerchantProperty.logger.debug(
                    "ConverterUtil :: convert json :: key:{} with value:{} is not a valid attribute".format(key, value))
                # raise SDKException.get_sdk_exception_with_json_data(
                #    ErrorConstants.ErrorMessage.JSONSTRING_TO_OBJECT_CONVERSION_FAILED, str(arg))
            self.__dict__[key] = value



from paytmpg.pg.response.BaseResponseBody import BaseResponseBody
from paytmpg.pg.utils.stringUtil import make_string, equals


class AsyncRefundResponseBody(BaseResponseBody):

    def __init__(self):
        self.refundId = None
        self.mid = None
        self.txnId = None
        self.orderId = None
        self.txnAmount = None
        self.refundAmount = None
        self.refId = None
        super(AsyncRefundResponseBody, self).__init__()

    def set_refund_id(self, refund_id):
        self.refundId = refund_id

    def get_refund_id(self):
        return self.refundId

    def set_mid(self, mid):
        self.mid = mid

    def get_mid(self):
        return self.mid

    def set_txn_id(self, txn_id):
        self.txnId = txn_id

    def get_txn_id(self):
        return self.txnId

    def set_order_id(self, order_id):
        self.orderId = order_id

    def get_order_id(self):
        return self.orderId

    def set_txn_amount(self, txn_amount):
        self.txnAmount = txn_amount

    def get_txn_amount(self):
        return self.txnAmount

    def set_refund_amount(self, refund_amount):
        self.refundAmount = refund_amount

    def get_refund_amount(self):
        return self.refundAmount

    def set_ref_id(self, ref_id):
        self.refId = ref_id

    def get_ref_id(self):
        return self.refId

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)



from paytmpg.pg.utils.stringUtil import make_string, equals


class NativeRefundStatusRequestBody:

    def __init__(self):
        self.mid = None
        self.orderId = None
        self.refId = None

    def set_mid(self, mid):
        self.mid = mid
        return self

    def get_mid(self):
        return self.mid

    def set_order_id(self, order_id):
        self.orderId = order_id
        return self

    def get_order_id(self):
        return self.orderId

    def set_ref_id(self, ref_id):
        self.refId = ref_id
        return self

    def get_ref_id(self):
        return self.refId

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)



"""Paytm Module
it comprises of merchant and payment gateway module
"""

from paytmpg.merchant.models.PaymentDetail import PaymentDetailsBuilder
from paytmpg.merchant.models.PaymentStatusDetail import PaymentStatusDetailBuilder
from paytmpg.merchant.models.RefundDetail import RefundDetailBuilder
from paytmpg.merchant.models.RefundStatusDetail import RefundStatusDetailBuilder

from paytmpg.pg.constants.MerchantProperty import MerchantProperty
from paytmpg.pg.constants.LibraryConstants import LibraryConstants

from paytmpg.pg.Payment import Payment
from paytmpg.pg.Refund import Refund

from paytmpg.pg.enums.EnumCurrency import EnumCurrency
from paytmpg.pg.enums.EChannelId import EChannelId
from paytmpg.pg.enums import UserSubWalletType

from paytmpg.pg.models.ExtendInfo import ExtendInfo
from paytmpg.pg.models.ShippingInfo import ShippingInfo
from paytmpg.pg.models.GoodsInfo import GoodsInfo
from paytmpg.pg.models.UserInfo import UserInfo
from paytmpg.pg.models.PaymentMode import PaymentMode
from paytmpg.pg.models.Money import Money

__all__ = [ExtendInfo, ShippingInfo, GoodsInfo, UserInfo, PaymentMode, Money, EChannelId, EnumCurrency,
           UserSubWalletType, LibraryConstants, MerchantProperty, Payment, Refund, PaymentDetailsBuilder,
           PaymentStatusDetailBuilder, RefundDetailBuilder, RefundStatusDetailBuilder]




from paytmpg.pg.request.BaseHeader import BaseHeader
from paytmpg.pg.utils.stringUtil import make_string, equals


class RequestHeader(BaseHeader):

    def __init__(self):
        super(RequestHeader, self).__init__()
        self.requestTimestamp = None

    def set_request_timestamp(self, request_timestamp):
        self.requestTimestamp = request_timestamp

    def get_request_timestamp(self):
        return self.requestTimestamp

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)





# dictionary for mapping value of type to UserWalletObject
type_user_map = dict()
# dictionary for mapping value of code to UserWalletObject
code_user_map = dict()


class UserSubWalletType:

    def __init__(self, code=None, type=None):
        self.code = code
        self.type = type
        type_user_map[self.type] = self
        code_user_map[self.code] = self

    def get_type(self):
        return self.type

    def set_type(self, type):
        self.type = type
        type_user_map[self.type] = self

    def get_code(self):
        return self.code

    def set_code(self, code):
        self.code = code
        code_user_map[self.code] = self

    # return object of type value == arg
    @staticmethod
    def get_ppi_type(arg):
        return type_user_map[arg]

    # return object of code value == arg
    @staticmethod
    def get_ppi_code(arg):
        return code_user_map[arg]

    def __str__(self):
        return '"' + self.type + '"'

    def __repr__(self):
        return '"' + self.type + '"'


FOOD = UserSubWalletType(2, "FOOD")
GIFT = UserSubWalletType(3, "GIFT")
MULTI_PURPOSE_GIFT = UserSubWalletType(4, "MULTI_PURPOSE_GIFT")
TOLL = UserSubWalletType(6, "TOLL")
CLOSED_LOOP_WALLET = UserSubWalletType(7, "CLOSED_LOOP_WALLET")
CLOSED_LOOP_SUB_WALLET = UserSubWalletType(8, "CLOSED_LOOP_SUB_WALLET")
FUEL = UserSubWalletType(9, "FUEL")
INTERNATIONAL_FUNDS_TRANSFER = UserSubWalletType(10, "INTERNATIONAL_FUNDS_TRANSFER")
CASHBACK = UserSubWalletType(11, "CASHBACK")
GIFT_VOUCHER = UserSubWalletType(12, "GIFT_VOUCHER")
COMMUNICATION = UserSubWalletType(13, "COMMUNICATION")



from paytmpg.pg.utils.stringUtil import make_string, equals


class UserInfo:

    def __init__(self):
        self.custId = None
        self.mobile = None
        self.email = None
        self.firstName = None
        self.lastName = None
        self.address = None
        self.pincode = None

    def set_cust_id(self, cust_id):
        self.custId = cust_id

    def get_cust_id(self):
        return self.custId

    def set_mobile(self, mobile):
        self.mobile = mobile

    def get_mobile(self):
        return self.mobile

    def set_email(self, email):
        self.email = email

    def get_email(self):
        return self.email

    def set_first_name(self, first_name):
        self.firstName = first_name

    def get_first_name(self):
        return self.firstName

    def set_last_name(self, last_name):
        self.lastName = last_name

    def get_last_name(self):
        return self.lastName

    def set_address(self, address):
        self.address = address

    def get_address(self):
        return self.address

    def set_pincode(self, pincode):
        self.pincode = pincode

    def get_pincode(self):
        return self.pincode

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)



from paytmpg.pg.request.RequestHeader import RequestHeader
from paytmpg.pg.utils.stringUtil import make_string, equals


class SecureRequestHeader(RequestHeader):

    def __init__(self):
        super(SecureRequestHeader, self).__init__()
        self.clientId = None
        self.signature = None

    def set_signature(self, signature):
        self.signature = signature

    def get_signature(self):
        return self.signature

    def set_client_id(self, client_id):
        self.clientId = client_id

    def get_client_id(self):
        return self.clientId

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)



from paytmpg.pg.response.BaseResponseBody import BaseResponseBody
from paytmpg.pg.utils.stringUtil import make_string, equals


class InitiateTransactionResponseBody(BaseResponseBody):

    def __init__(self):
        super(InitiateTransactionResponseBody, self).__init__()
        self.txnToken = None
        self.isPromoCodeValid = None
        self.authenticated = None
        self.callbackUrl = None

    def set_txn_token(self, txn_token):
        self.txnToken = txn_token

    def get_txn_token(self):
        return self.txnToken

    def set_is_promo_code_valid(self, is_promo_code_valid):
        self.isPromoCodeValid = is_promo_code_valid

    def get_is_promo_code_valid(self):
        return self.isPromoCodeValid

    def set_is_authenticated(self, is_authenticated):
        self.authenticated = is_authenticated

    def get_is_authenticated(self):
        return self.authenticated

    def set_callback_url(self, callback_url):
        self.callbackUrl = callback_url

    def get_callback_url(self):
        return self.callbackUrl

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)



from requests import post

from paytmpg.pg.constants.MerchantProperty import MerchantProperty
from paytmpg.pg.constants.LibraryConstants import LibraryConstants

from paytmpg.pg.utils.ConverterUtil import JsonToObject
from paytmpg.pg.utils.SignatureUtil import verifySignature
from paytmpg.pg.SDKException import SDKException
from paytmpg.pg.utils.stringUtil import is_empty, format_string


class Request:

    headers = {'Content-type': LibraryConstants.APPLICATION_JSON_TEXT}

    @classmethod
    def process(cls, request, url, response, read_timeout=30):
        """
        :param request: it contain head and body which we want to transmit over network
        :param url: on which we want to make hit
        :param response: object in which value of response will be putted
        :param read_timeout: this is in "Second" unit
        :return: None, it is just setting value in response object
        """
        cls.headers[LibraryConstants.X_REQUEST_ID] = MerchantProperty.request_id
        post_args = {
            'headers': cls.headers,
            'timeout': (MerchantProperty.connect_timeout, read_timeout),
            'data': format_string(request)
        }
        res = post(url, **post_args)
        MerchantProperty.logger.info("Request :: do_transaction for request: {} ".format(format_string(request)))
        MerchantProperty.logger.info("Request :: do_transaction response for request: {} ".format(res))
        MerchantProperty.logger.info("Request :: do_transaction response content for request: {} ".format(res.content))
        response.set_json_response(res.content)
        if res.status_code == "500" or res.status_code == "400":
            MerchantProperty.logger.info(
                "Request :: do_transaction for request: status code {}".format(res.status_code))
            return SDKException.get_sdk_exception(str(res.content))
        str_body = cls.__get_body_from_response_content(str(res.content))
        request_response = res.json()
        JsonToObject(request_response, response.get_response_object())
        result_info = response.get_response_object().get_body().get_result_info()
        if response.get_response_object().get_head().get_signature() is not None and (
                LibraryConstants.SUCCESS_STATUS == result_info.get_result_status()
                or LibraryConstants.TXN_SUCCESS_STATUS == result_info.get_result_status()
                or LibraryConstants.PENDING_STATUS == result_info.get_result_status()) and \
                not verifySignature(str_body, MerchantProperty.get_merchant_key(),
                                    response.get_response_object().get_head().get_signature()):
            raise SDKException.get_signature_validation_failed_exception(res.content)

    @staticmethod
    def __get_body_from_response_content(content):
        if is_empty(content) or ('"body":{' not in content) or ('"head":{' not in content):
            raise SDKException.get_sdk_exception("Received response body is not proper.")
        MerchantProperty.logger.debug("Request :: get_body_from_response_content content: {}".format(content))
        head_idx = content.find(LibraryConstants.HEAD_TEXT, 0, len(content))
        start_idx, end_idx = content.find(LibraryConstants.BODY_TEXT, 0, len(content)) + 6, len(
            content) - (2 if content.endswith('\'') else 1)
        if head_idx > start_idx:
            end_idx = head_idx-2
        MerchantProperty.logger.debug(
            "Request :: get_body_from_response_content body: {}".format(content[start_idx:end_idx]))
        return content[start_idx:end_idx]



from paytmpg.pg.utils.stringUtil import make_string, equals


class ExtraParameterMap(object):

    def __init__(self):
        # str to object
        self.extraParamsMap = None

    def get_extra_parameter_map(self):
        return self.extraParamsMap

    def set_extra_parameter_map(self, extra_param_map):
        self.extraParamsMap = extra_param_map
        return self

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)




from paytmpg.pg.enums import UserSubWalletType
from paytmpg.pg.request.RefundBaseRequest import RefundBaseRequest
from paytmpg.pg.utils.stringUtil import make_string, equals
from paytmpg.pg.utils.ConverterUtil import JsonToObject


class RefundInitiateRequestBody(RefundBaseRequest):

    def __init__(self):
        super(RefundInitiateRequestBody, self).__init__()
        self.txnId = None
        self.refundAmount = None
        self.comments = None
        self.txnType = None
        self.preferredDestination = None
        self.requestId = None
        # map of [UserSubWalletType to big_decimal]
        self.subwalletAmount = None

    # use decorator for this get

    def set_txn_id(self, txn_id):
        self.txnId = txn_id

    def get_txn_id(self):
        return self.txnId

    def set_refund_amount(self, refund_amount):
        self.refundAmount = refund_amount

    def get_refund_amount(self):
        return self.refundAmount

    def set_comments(self, comments):
        self.comments = comments

    def get_comments(self):
        return self.comments

    def set_txn_type(self, txn_type):
        self.txnType = txn_type

    def get_txn_type(self):
        return self.txnType

    def set_preferred_destination(self, preferred_destination):
        self.preferredDestination = preferred_destination

    def get_preferred_destination(self):
        return self.preferredDestination

    def set_request_id(self, request_id):
        self.requestId = request_id

    def get_request_id(self):
        return self.requestId

    def set_sub_wallet_amount(self, sub_wallet_amount):
        self.subwalletAmount = sub_wallet_amount

    def get_sub_wallet_amount(self):
        return self.subwalletAmount

    def set_sub_wallet_amount_string_map(self, sub_wallet_amount_string_dict):
        if sub_wallet_amount_string_dict is not None and len(sub_wallet_amount_string_dict) != 0:
            self.subwalletAmount = dict(UserSubWalletType(), int)
            JsonToObject(sub_wallet_amount_string_dict, self.subwalletAmount)

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)



from uuid import uuid4
from paytmpg.pg.constants.ErrorConstants import ErrorConstants
from paytmpg.pg.constants.LibraryConstants import LibraryConstants
from paytmpg.pg.constants.MerchantProperty import MerchantProperty

from paytmpg.pg.request.SecureRequestHeader import SecureRequestHeader
from paytmpg.pg.response.ResultInfo import ResultInfo
from paytmpg.merchant.models.Time import current_time_millis
from paytmpg.pg.SDKException import SDKException
from paytmpg.merchant.models.SDKResponse import SDKResponse


class CommonUtil:
    """
        This class receive the Paytm response objects and translate the to their
        respective merchant response objects and returns call to Merchant Application
    """

    @staticmethod
    def get_secure_request_header(client_id, channel_id=None):
        """This will make secure header
        :param client_id: client_id of merchant
        :param channel_id: channel id from which call is made WEB or WAP
        :return: SecureRequestHeader object
        """
        head = SecureRequestHeader()
        head.set_version(LibraryConstants.VERSION)
        head.set_channel_id(channel_id)
        head.set_request_timestamp(str(current_time_millis()))
        head.set_client_id(client_id)
        return head

    @staticmethod
    def get_sdk_response(e, obj):
        """
        :param e: Exception_response body
        :param obj: object of response body which is to be set with result_info
        :return: SDKResponse {raw_data, response obj}
        """
        MerchantProperty.logger.critical("CommonUtil :: get_sdk_response ExceptionResponseBody: {}".format(e))
        MerchantProperty.logger.debug("CommonUtil :: get_sdk_response type of e is {}".format(type(e)))
        result = ResultInfo()
        result.set_result_status(ErrorConstants.FAILURE)
        result.set_result_code(ErrorConstants.ErrorCode.DEFAULT)
        if isinstance(e, SDKException):
            result.set_result_msg(e.get_msg())
        else:
            result.set_result_msg(e)
        obj.get_body().set_result_info(result)
        if isinstance(e, SDKException):
            MerchantProperty.logger.debug("CommonUtil :: get_sdk_response obj: {}".format(obj))
            return SDKResponse().set_response_object(obj).set_json_response(e.get_raw_data())
        return SDKResponse().set_response_object(obj)

    @staticmethod
    def get_unique_request_id():
        return LibraryConstants.PYTHON_SDK_TEXT + ":" + LibraryConstants.PYTHON_SDK_VERSION + ":" + str(uuid4())



from paytmpg.pg.utils.stringUtil import make_string, equals


class Money:

    def __init__(self, currency, value):
        self.currency = currency
        self.value = value

    def set_currency(self, currency):
        self.currency = currency

    def set_value(self, value):
        self.value = value

    def get_curency(self):
        return self.currency

    def get_value(self):
        return self.value

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)



from paytmpg.pg.constants.MerchantProperty import MerchantProperty
from paytmpg.pg.request.NativeRefundStatusRequestBody import NativeRefundStatusRequestBody
from paytmpg.pg.utils.stringUtil import make_string, equals


class RefundStatusDetail:

    """Refund status Parameters"""
    def __init__(self, refund_status_detail_builder):
        """Unique order Id"""
        self.__order_id = refund_status_detail_builder.order_id
        """Unique ref id for each refund request"""
        self.__ref_id = refund_status_detail_builder.ref_id
        """Read TimeOut for Refund status Api """
        self.__read_timeout = refund_status_detail_builder.read_timeout

    def get_order_id(self):
        return self.__order_id

    def get_ref_id(self):
        return self.__ref_id

    def get_read_timeout(self):
        return self.__read_timeout

    def __str__(self):
        return ""

    def __eq__(self, other):
        return equals(self)

    def __dir__(self):
        return ""

    def __create_native_refund_status_request_body(self):
        return NativeRefundStatusRequestBody().set_mid(MerchantProperty.get_mid()).set_order_id(
            self.get_order_id()).set_ref_id(self.get_ref_id())


class RefundStatusDetailBuilder:
    """
    RefundStatusDetailBuilder is used to build the Refund Status object
    """
    order_id = None
    ref_id = None
    """in second """
    read_timeout = 5 * 60

    def __init__(self):
        pass

    def set_order_id(self, order_id):
        self.order_id = order_id
        return self

    def set_ref_id(self, ref_id):
        self.ref_id = ref_id
        return self

    def set_read_timeout(self, read_timeout):
        self.read_timeout = read_timeout
        return self

    def build(self):
        return RefundStatusDetail(self)

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self)

    def __setattr__(self, key, value):
        if not hasattr(self, key):
            raise AttributeError(" Can not be set ".format(key))
        self.__dict__[key] = value

    def __dir__(self):
        return ""



from paytmpg.pg.utils.stringUtil import make_string, equals


class BaseHeader(object):

    def __init__(self):
        self.version = None
        self.channelId = None

    def set_version(self, version):
        self.version = version

    def get_version(self):
        return self.version

    def get_channel_id(self):
        return self.channelId

    def set_channel_id(self, channel_id):
        self.channelId = channel_id

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)





"""Model module
"""


from paytmpg.pg.models import ExtendInfo
from paytmpg.pg.models import GoodsInfo
from paytmpg.pg.models import Money
from paytmpg.pg.models import PaymentMode
from paytmpg.pg.models import ShippingInfo
from paytmpg.pg.models import UserInfo

__all__ = [ExtendInfo, GoodsInfo, Money, PaymentMode, ShippingInfo, UserInfo]



enum_currency = dict()


class EnumCurrency:

    def __init__(self, currency="INR"):
        self.currency = currency
        enum_currency[currency] = self

    def get_currency(self):
        return self.currency

    @staticmethod
    def get_enum_by_currency(currency):
        return enum_currency[currency]

    INR = "INR"



from paytmpg.pg.utils.stringUtil import make_string, equals


class PaymentMode:

    def __init__(self, mode):
        self.channels = None
        self.mode = mode

    def get_mode(self):
        return self.mode

    def set_mode(self, mode):
        self.mode = mode

    def set_channels(self, channels):
        self.channels = channels

    def get_channels(self):
        return self.channels

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)

    def __repr__(self):
        return make_string(self)



from sys import exc_info

from paytmpg.pg.request.InitiateTransactionRequest import InitiateTransactionRequest
from paytmpg.pg.request.NativePaymentStatusRequest import NativePaymentStatusRequest

from paytmpg.pg.response.InitiateTransactionResponse import InitiateTransactionResponse
from paytmpg.pg.response.NativePaymentStatusResponse import NativePaymentStatusResponse

from paytmpg.pg.utils.CommonUtil import CommonUtil
from paytmpg.pg.utils.SignatureUtil import generateSignature
from paytmpg.pg.utils.stringUtil import is_empty, format_string

from paytmpg.pg.constants.MerchantProperty import MerchantProperty

from paytmpg.pg.SDKException import SDKException
from paytmpg.pg.Request import Request
from paytmpg.merchant.models.SDKResponse import SDKResponse


class Payment:
    """This class handle all call(which is from DemoApp) related to create transaction and payment status  
    """
    @classmethod
    def createTxnToken(cls, payment_details):
        """
        :param payment_details: has all mandatory parameter and some others
        :return: SDKResponse {jsonData, InitiateTransactionResponse}
        """
        MerchantProperty.set_logger_name(CommonUtil.get_unique_request_id())
        try:
            if not MerchantProperty.is_initialized:
                return CommonUtil.get_sdk_response(SDKException.get_merchant_property_initialization_exception(), InitiateTransactionResponse())
            MerchantProperty.logger.info("Payment :: create_txn_token payment_details: {}".format(payment_details))
            cls.__validate_create_txn_token(payment_details)
            request = cls.__create_initiate_transaction_request(payment_details)
            cls.__set_signature(request.get_head(), request.get_body())
            response = SDKResponse().set_response_object(InitiateTransactionResponse())
            url = MerchantProperty.get_initiate_txn_url() + '?' + "mid=" + request.get_body().get_mid() + \
                "&orderId=" + request.get_body().get_order_id()
            Request.process(request, url, response, payment_details.get_read_timeout())
            return response
        except Exception:
            return CommonUtil.get_sdk_response(exc_info()[1], InitiateTransactionResponse())

    @staticmethod
    def __create_initiate_transaction_request(payment_details):
        """This method convert payment_details into InitiateTransactionRequestBody()
        :param payment_details: has all mandatory parameter and some others
        :return: InitiateTransactionRequest object
        """
        head = CommonUtil.get_secure_request_header(MerchantProperty.get_client_id(), payment_details.get_channel_id())
        body = payment_details._PaymentDetail__create_initiate_transaction_request_body()
        return InitiateTransactionRequest().set_head(head).set_body(body)

    @staticmethod
    def __set_signature(head, body):
        """
        :param head: secure-header
        :param body: body of which we are making checksum and adding to head
        :return:
        """
        signature = generateSignature(format_string(body), MerchantProperty.get_merchant_key())
        head.set_signature(signature)

    @staticmethod
    def __validate_create_txn_token(payment_details):
        """Checking mandatory field it has or not
        :param payment_details:
        :return: if mandatory parameter is not there return SDKException
        """
        MerchantProperty.logger.info("Payment :: validateCreateTxnToken for payment_details: {0} ".format(payment_details))
        if is_empty(payment_details.get_order_id()) or not payment_details.get_txn_amount()\
                or is_empty(payment_details.get_txn_amount().get_value()) or not payment_details.get_user_info()\
                or is_empty(payment_details.get_user_info().get_cust_id()):
            raise SDKException.get_missing_mandatory_parameters_exception()

    @staticmethod
    def __create_native_payment_status_request(payment_status_detail):
        """
        :param payment_status_detail: has all mandatory parameter and some others
        :return: NativePaymentStatusRequest object
        """
        head = CommonUtil.get_secure_request_header(MerchantProperty.get_client_id(), None)
        body = payment_status_detail._PaymentStatusDetail__create_native_payment_status_request_body()
        request = NativePaymentStatusRequest()
        request.set_head(head)
        request.set_body(body)
        return request

    @classmethod
    def getPaymentStatus(cls, payment_status_detail):
        """
        :param payment_status_detail: it has all parameter to make request
        :return: SDKResponse { json response data, NativePaymentStatusRequest object}
        """
        MerchantProperty.set_logger_name(CommonUtil.get_unique_request_id())
        try:
            if not MerchantProperty.is_initialized:
                return CommonUtil.get_sdk_response(SDKException.get_merchant_property_initialization_exception(),
                                                   NativePaymentStatusRequest())
            MerchantProperty.logger.info(
                "Payment :: get_payment_status payment_status_detail: {}".format(payment_status_detail))
            cls.__validate_get_payment_status(payment_status_detail)
            request = cls.__create_native_payment_status_request(payment_status_detail)
            cls.__set_signature(request.get_head(), request.get_body())
            response = SDKResponse().set_response_object(NativePaymentStatusResponse())
            Request.process(request, MerchantProperty.get_payment_status_url(), response
                            , payment_status_detail.get_read_timeout())
            return response
        except Exception:
            return CommonUtil.get_sdk_response(exc_info()[1], NativePaymentStatusResponse())

    @staticmethod
    def __validate_get_payment_status(payment_status_detail):
        """
        :param payment_status_detail:
        :return: will raise exception if there is missing of any mandatory parameter
        """
        MerchantProperty.logger.info(
            "Payment :: validate_get_payment_status for payment_status_detail: {}".format(payment_status_detail))
        if is_empty(payment_status_detail.get_order_id()):
            raise SDKException.get_missing_mandatory_parameters_exception()

    def __dir__(self):
        return ""



try:
    import configparser
except ImportError:
    import ConfigParser as configparser

from sys import version_info
from os.path import abspath


class LibraryConstants:
    """ Below constants are used in API calling
    """
    VERSION = "v2"
    DATE_FORMAT = "yyyy.MM.dd.HH.mm.ss z"
    STAGING_ENVIRONMENT = "STAGE"
    PRODUCTION_ENVIRONMENT = "PROD"

    """Status message can be returned in case of Api Success"""
    SUCCESS_STATUS = "S"
    PENDING_STATUS = "PENDING"
    TXN_SUCCESS_STATUS = "TXN_SUCCESS"
    """Test String used in API callings"""
    TRUE_TEXT = 'true'
    FALSE_TEXT = 'false'
    MID_TEXT = 'mid'
    ORDER_ID_TEXT = 'orderId'
    CONTENT_TYPE_TEXT = 'Content-Type'
    APPLICATION_JSON_TEXT = 'application/json'
    UTF_8_TEXT = 'UTF-8'
    SUCCESS_TEXT = 'SUCCESS'
    HEAD_TEXT = 'head'
    BODY_TEXT = 'body'
    SIGNATURE_TEXT = 'signature'
    PYTHON_SDK_TEXT = "PYTHON-SDK"
    X_REQUEST_ID = "X-Request-ID"
    """below text as these are used input name for redirection
    flow in process transaction api calling"""
    MID_TEXT_DEFAULT = 'MID'
    WEBSITE_TEXT_DEFAULT = 'WEBSITE'
    CALLBACK_URL_TEXT_DEFAULT = 'CALLBACK_URL'
    CHECKSUM_HASH_TEXT_DEFAULT = 'CHECKSUMHASH'

    # this jsp is used in redirection flow
    DEFAULT_REDIRECT_JSP = "pgRedirect.jsp"
    REQUEST_TYPE_PAYMENT = "Payment"
    MEDIA_TYPE_JSON = "application/json; charset=utf-8"

    config = configparser.ConfigParser()
    file_path = '/'.join(abspath(__file__).split('/')[:-4])
    file_abs_path = file_path + "/VERSION.ini"
    config.read(file_abs_path)
    if version_info[0] < 3:
        PYTHON_SDK_VERSION = config.get('VERSION_INFO', 'package_version')
    else:
        PYTHON_SDK_VERSION = config['VERSION_INFO']['package_version']

    class Request:
        FOOD_WALLET = "FOOD"
        GIFT_WALLET = "GIFT"



from paytmpg.pg.utils.stringUtil import make_string, equals


class ShippingInfo:

    def __init__(self):
        self.merchantShippingId = None
        self.trackingNo = None
        self.carrier = None
        self.chargeAmount = None
        self.countryName = None
        self.stateName = None
        self.cityName = None
        self.address1 = None
        self.address2 = None
        self.firstName = None
        self.lastName = None
        self.mobileNo = None
        self.zipCode = None
        self.email = None

    def set_merchant_shipping_id(self, merchant_shipping_id):
        self.merchantShippingId = merchant_shipping_id

    def get_merchant_shipping_id(self):
        return self.merchantShippingId

    def set_tracking_no(self, tracking_no):
        self.trackingNo = tracking_no

    def get_tracking_no(self):
        return self.trackingNo

    def set_carrier(self, carrier):
        self.carrier = carrier

    def get_carrier(self):
        return self.carrier

    def set_charge_amount(self, charge_amount):
        self.chargeAmount = charge_amount

    def get_charge_amount(self):
        return self.chargeAmount

    def set_country_name(self, country_name):
        self.countryName = country_name

    def get_country_name(self):
        return self.countryName

    def set_state_name(self, state_name):
        self.stateName = state_name

    def get_state_name(self):
        return self.stateName

    def set_city_name(self, city_name):
        self.cityName = city_name

    def get_city_name(self):
        return self.cityName

    def set_address1(self, address1):
        self.address1 = address1

    def get_address1(self):
        return self.address1

    def set_address2(self, address2):
        self.address2 = address2

    def get_address2(self):
        return self.address2

    def set_first_name(self, first_name):
        self.firstName = first_name

    def get_first_name(self):
        return self.firstName

    def set_last_name(self, last_name):
        self.lastName = last_name

    def get_last_name(self):
        return self.lastName

    def set_mobile_no(self, mobile_no):
        self.mobileNo = mobile_no

    def get_mobile_no(self):
        return self.mobileNo

    def set_zip_code(self, zip_code):
        self.zipCode = zip_code

    def get_zip_code(self):
        return self.zipCode

    def set_email(self, email):
        self.email = email

    def get_email(self):
        return self.email

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)

    def __repr__(self):
        return make_string(self)



from paytmpg.pg.utils.stringUtil import make_string, equals


class InitiateTransactionRequestBody:

    def __init__(self):
        self.requestType = None
        self.mid = None
        self.orderId = None
        self.websiteName = None
        self.txnAmount = None
        self.userInfo = None
        self.paytmSsoToken = None
        self.enablePaymentMode = None
        self.disablePaymentMode = None
        self.promoCode = None
        self.callbackUrl = None
        self.goods = None
        self.shippingInfo = None
        self.extendInfo = None
        self.emiOption = None
        self.cardTokenRequired = None
        self.cartValidationRequired = None

    def set_request_type(self, request_type):
        self.requestType = request_type

    def get_request_type(self):
        return self.requestType

    def set_mid(self, mid):
        self.mid = mid

    def get_mid(self):
        return self.mid

    def set_order_id(self, order_id):
        self.orderId = order_id

    def get_order_id(self):
        return self.orderId

    def set_website_name(self, website_name):
        self.websiteName = website_name

    def get_website_name(self):
        return self.websiteName

    def set_txn_amount(self, txn_amount):
        self.txnAmount = txn_amount

    def get_txn_amount(self):
        return self.txnAmount

    def set_user_info(self, user_info):
        self.userInfo = user_info

    def get_user_info(self):
        return self.userInfo

    def set_paytm_sso_token(self, paytm_sso_token):
        self.paytmSsoToken = paytm_sso_token

    def get_paytm_sso_token(self):
        return self.paytmSsoToken

    def set_enable_payment_mode(self, enable_payment_mode):
        self.enablePaymentMode = enable_payment_mode

    def get_enable_payment_mode(self):
        return self.enablePaymentMode

    def set_disable_payment_mode(self, disable_payment_mode):
        self.disablePaymentMode = disable_payment_mode

    def get_disable_payment_mode(self):
        return self.disablePaymentMode

    def set_promo_code(self, promo_code):
        self.promoCode = promo_code

    def get_promo_code(self):
        return self.promoCode

    def set_callback_url(self, callback_url):
        self.callbackUrl = callback_url

    def get_callback_url(self):
        return self.callbackUrl

    def set_goods(self, goods):
        self.goods = goods

    def get_goods(self):
        return self.goods

    def set_shipping_info(self, shipping_info):
        self.shippingInfo = shipping_info

    def get_shipping_info(self):
        return self.shippingInfo

    def set_extend_info(self, extend_info):
        self.extendInfo = extend_info

    def get_extend_info(self):
        return self.extendInfo

    def set_emi_option(self, emi_option):
        self.emiOption = emi_option

    def get_emi_option(self):
        return self.emiOption

    def set_card_token_required(self, card_token_required):
        self.cardTokenRequired = card_token_required

    def get_card_token_required(self):
        return self.cardTokenRequired

    def set_cart_validation_required(self, cart_validation_required):
        self.cartValidationRequired = cart_validation_required

    def get_cart_validation_required(self):
        return self.cartValidationRequired

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)



from paytmpg.pg.utils.stringUtil import make_string, equals


class ExtendInfo:

    def __init__(self):
        self.udf1 = None
        self.udf2 = None
        self.udf3 = None
        self.mercUnqRef = None
        self.comments = None
        self.amountToBeRefunded = None
        self.subwalletAmount = None

    def set_udf1(self, udf1):
        self.udf1 = udf1

    def get_udf1(self):
        return self.udf1

    def set_udf2(self, udf2):
        self.udf2 = udf2

    def get_udf2(self):
        return self.udf2

    def set_udf3(self, udf3):
        self.udf3 = udf3

    def get_udf3(self):
        return self.udf3

    def set_merc_unq_ref(self, merc_unq_ref):
        self.mercUnqRef = merc_unq_ref

    def get_merc_unq_ref(self):
        return self.mercUnqRef

    def set_comments(self, comments):
        self.comments = comments

    def get_comments(self):
        return self.comments

    def set_amount_to_be_refunded(self, amount_to_be_refunded):
        self.amountToBeRefunded = amount_to_be_refunded

    def get_amount_to_be_refunded(self):
        return self.amountToBeRefunded

    def set_sub_wallet_amount(self, sub_wallet_amount):
        self.subwalletAmount = str()
        is_first = True
        for k, v in sub_wallet_amount.items():
            if not is_first:
                self.subwalletAmount += ','
            self.subwalletAmount += '"{}":"{}"'.format(k, v)
            is_first = False
        if is_first:
            self.subwalletAmount = None
        else:
            self.subwalletAmount = '{' + self.subwalletAmount + '}'

    def get_sub_wallet_amount(self):
        return self.subwalletAmount

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)



from paytmpg.pg.constants.MerchantProperty import MerchantProperty
from paytmpg.pg.constants.LibraryConstants import LibraryConstants
from paytmpg.pg.request.InitiateTransactionRequestBody import InitiateTransactionRequestBody
from paytmpg.pg.utils.stringUtil import make_string, equals


class PaymentDetail:

    def __init__(self, payment_detail_builder):
        self.__channelId = payment_detail_builder.channelId
        self.__orderId = payment_detail_builder.orderId
        self.__txnAmount = payment_detail_builder.txnAmount
        self.__userInfo = payment_detail_builder.userInfo
        self.__paytmSsoToken = payment_detail_builder.paytmSsoToken
        self.__enablePaymentMode = payment_detail_builder.enablePaymentMode
        self.__disablePaymentMode = payment_detail_builder.disablePaymentMode
        self.__goods = payment_detail_builder.goods
        self.__shippingInfo = payment_detail_builder.shippingInfo
        self.__extendInfo = payment_detail_builder.extendInfo
        self.__promoCode = payment_detail_builder.promoCode
        self.__emiOption = payment_detail_builder.emiOption
        self.__cardTokenRequired = payment_detail_builder.cardTokenRequired
        self.__readTimeout = payment_detail_builder.readTimeout

    def get_channel_id(self):
        return self.__channelId

    def get_order_id(self):
        return self.__orderId

    def get_txn_amount(self):
        return self.__txnAmount

    def get_user_info(self):
        return self.__userInfo

    def get_paytm_sso_token(self):
        return self.__paytmSsoToken

    def get_enable_payment_mode(self):
        return self.__enablePaymentMode

    def get_disable_payment_mode(self):
        return self.__disablePaymentMode

    def get_goods(self):
        return self.__goods

    def get_shipping_info(self):
        return self.__shippingInfo

    def get_promo_code(self):
        return self.__promoCode

    def get_extend_info(self):
        return self.__extendInfo

    def get_emi_option(self):
        return self.__emiOption

    def get_card_token_required(self):
        return self.__cardTokenRequired

    def get_read_timeout(self):
        return self.__readTimeout

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)

    def __dir__(self):
        return ""

    def __create_initiate_transaction_request_body(self):
        body = InitiateTransactionRequestBody()

        body.set_request_type(LibraryConstants.REQUEST_TYPE_PAYMENT)
        body.set_mid(MerchantProperty.get_mid())
        body.set_website_name(MerchantProperty.get_website())
        body.set_callback_url(MerchantProperty.get_callback_url())

        body.set_order_id(self.get_order_id())
        body.set_txn_amount(self.get_txn_amount())
        body.set_user_info(self.get_user_info())
        body.set_paytm_sso_token(self.get_paytm_sso_token())
        body.set_enable_payment_mode(self.get_enable_payment_mode())
        body.set_disable_payment_mode(self.get_disable_payment_mode())
        body.set_promo_code(self.get_promo_code())
        body.set_goods(self.get_goods())
        body.set_shipping_info(self.get_shipping_info())
        body.set_extend_info(self.get_extend_info())
        body.set_emi_option(self.get_emi_option())
        body.set_card_token_required(self.get_card_token_required())
        return body


class PaymentDetailsBuilder:

    channelId = None
    orderId = None
    txnAmount = None
    userInfo = None
    paytmSsoToken = None
    enablePaymentMode = None
    disablePaymentMode = None
    goods = None
    shippingInfo = None
    promoCode = None
    extendInfo = None
    emiOption = None
    cardTokenRequired = None
    readTimeout = None

    def __init__(self, channel_id, order_id, txn_amount, user_info):
        self.channelId = channel_id
        self.orderId = order_id
        self.txnAmount = txn_amount
        self.userInfo = user_info
        self.readTimeout = 30

    def build(self):
        return PaymentDetail(self)

    def set_channel_id(self, channel_id):
        self.channelId = channel_id
        return self

    def set_order_id(self, order_id):
        self.orderId = order_id
        return self

    def set_txn_amount(self, txn_amount):
        self.txnAmount = txn_amount
        return self

    def set_user_info(self, user_info):
        self.userInfo = user_info
        return self

    def set_paytm_sso_token(self, paytm_sso_token):
        self.paytmSsoToken = paytm_sso_token
        return self

    def set_enable_payment_mode(self, enable_payment_mode):
        self.enablePaymentMode = enable_payment_mode
        return self

    def set_disable_payment_mode(self, disable_payment_mode):
        self.disablePaymentMode = disable_payment_mode
        return self

    def set_goods(self, goods):
        self.goods = goods
        return self

    def set_shipping_info(self, shipping_info):
        self.shippingInfo = shipping_info
        return self

    def set_promo_code(self, promo_code):
        self.promoCode = promo_code
        return self

    def set_extend_info(self, extend_info):
        self.extendInfo = extend_info
        return self

    def set_emi_option(self, emi_option):
        self.emiOption = emi_option
        return self

    def set_card_token_required(self, card_token_required):
        self.cardTokenRequired = card_token_required
        return self

    def set_read_timeout(self, read_timeout):
        self.readTimeout = read_timeout
        return self

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)

    def __setattr__(self, key, value):
        if not hasattr(self, key):
            raise AttributeError(" Can not be set ".format(key))
        self.__dict__[key] = value



from paytmpg.pg.response.BaseResponseBody import BaseResponseBody
from paytmpg.pg.utils.stringUtil import make_string, equals


class NativeRefundStatusResponseBody(BaseResponseBody):

    def __init__(self):
        super(NativeRefundStatusResponseBody, self).__init__()
        self.txnId = None
        self.orderId = None
        self.txnAmount = None
        self.mid = None
        self.refundAmout = None
        self.txnDate = None
        self.totalRefundAmount = None
        self.refundDate = None
        self.refId = None
        self.bankTxnId = None
        self.txnType = None
        self.gateway = None
        self.bankName = None
        self.paymentMode = None
        self.refundId = None
        self.refundType = None
        self.ssoId = None

    def get_txn_id(self):
        return self.txnId

    def get_order_id(self):
        return self.orderId

    def get_txn_amount(self):
        return self.txnAmount

    def get_mid(self):
        return self.mid

    def get_refund_amount(self):
        return self.refundAmout

    def get_txn_date(self):
        return self.txnDate

    def get_total_refunded_amount(self):
        return self.totalRefundAmount

    def get_refund_date(self):
        return self.refundDate

    def get_ref_id(self):
        return self.refId

    def get_bank_txn_id(self):
        return self.bankTxnId

    def get_txn_type(self):
        return self.txnType

    def get_gateway_name(self):
        return self.gateway

    def get_bank_name(self):
        return self.bankName

    def get_payment_mode(self):
        return self.paymentMode

    def get_refund_id(self):
        return self.refundId

    def get_refund_type(self):
        return self.refundType

    def get_sso_id(self):
        return self.ssoId

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)



from paytmpg.pg.constants.MerchantProperty import MerchantProperty
from paytmpg.pg.request.NativePaymentStatusRequestBody import NativePaymentStatusRequestBody
from paytmpg.pg.utils.stringUtil import make_string, equals


class PaymentStatusDetail:

    def __init__(self, payment_status_detail_builder):
        self.__orderId = payment_status_detail_builder.order_id
        self.__readTimeout = payment_status_detail_builder.read_timeout

    def get_order_id(self):
        return self.__orderId

    def get_read_timeout(self):
        return self.__readTimeout

    def __str__(self):
        return ""

    def __eq__(self, other):
        return equals(self)

    def __dir__(self):
        return ""

    def __create_native_payment_status_request_body(self):
        native_payment_status_request_body = NativePaymentStatusRequestBody()
        native_payment_status_request_body.set_mid(MerchantProperty.get_mid())
        native_payment_status_request_body.set_order_id(self.get_order_id())
        return native_payment_status_request_body


class PaymentStatusDetailBuilder:
    order_id = None
    read_timeout = None

    def __init__(self, order_id):
        self.order_id = order_id
        self.read_timeout = 30

    def build(self):
        return PaymentStatusDetail(self)

    def set_read_timeout(self, read_timeout):
        self.read_timeout = read_timeout
        return self

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self)

    def __setattr__(self, key, value):
        if not hasattr(self, key):
            raise AttributeError(" Can not be set ".format(key))
        self.__dict__[key] = value

    def __dir__(self):
        return ""



from paytmpg.pg.request.ExtraParameterMap import ExtraParameterMap
from paytmpg.pg.response.ResultInfo import ResultInfo
from paytmpg.pg.utils.stringUtil import make_string, equals


class BaseResponseBody(ExtraParameterMap):

    def __init__(self):
        self.resultInfo = ResultInfo()
        super(BaseResponseBody, self).__init__()

    def get_result_info(self):
        return self.resultInfo

    def set_result_info(self, result_info):
        self.resultInfo = result_info

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)



from paytmpg.pg.utils.stringUtil import make_string, equals


class ChildTransaction:

    def __init__(self):
        self.txnId = None
        self.paymentMode = None
        self.txnAmount = None
        self.gateway = None
        self.bankTxnId = None
        self.bankName = None
        self.status = None
        self.cardIndexNo = None
        self.maskedCardNo = None

    def set_txn_id(self, txn_id):
        self.txnId = txn_id

    def get_txn_id(self):
        return self.txnId

    def set_payment_mode(self, payment_mode):
        self.paymentMode = payment_mode

    def get_payment_mode(self):
        return self.paymentMode

    def set_txn_amount(self, txn_amount):
        self.txnAmount = txn_amount

    def get_txn_amount(self):
        return self.txnAmount

    def set_gateway(self, gateway):
        self.gateway = gateway

    def get_gateway(self):
        return self.gateway

    def set_bank_txn_id(self, bank_txn_id):
        self.bankTxnId = bank_txn_id

    def get_bank_txn_id(self):
        return self.bankTxnId

    def set_bank_name(self, bank_name):
        self.bankName = bank_name

    def get_bank_name(self):
        return self.bankName

    def set_status(self, status):
        self.status = status

    def get_status(self):
        return self.status

    def set_card_index_no(self, card_index_no):
        self.cardIndexNo = card_index_no

    def get_card_index_no(self):
        return self.cardIndexNo

    def set_masked_card_no(self, masked_card_no):
        self.maskedCardNo = masked_card_no

    def get_masked_card_no(self):
        return self.maskedCardNo

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)



from setuptools import find_packages, setup
try:
    import configparser
except ImportError:
    import ConfigParser as configparser
from sys import version_info

long_description = "Python sdk "
config = configparser.ConfigParser()
config.read("VERSION.ini")

if version_info[0] < 3:
    PYTHON_SDK_VERSION = config.get('VERSION_INFO', 'package_version')
else:
    PYTHON_SDK_VERSION = config['VERSION_INFO']['package_version']

setup(
    name='paytm-pg',
    version=PYTHON_SDK_VERSION,
    author='avneesh gupta',
    author_email='pgplus.tyche@paytm.com',
    description="Merchant util library which provide payment, payment status, refund and refund status",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/paytm/paytm-pg-python-sdk",
    packages=find_packages(),
    install_requires=["requests", "pycryptodome"],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
)



from paytmpg.pg.utils.stringUtil import make_string, equals


class NativeRefundStatusRequest:

    def __init__(self):
        self.head = None
        self.body = None

    def set_head(self, head):
        self.head = head
        return self

    def get_head(self):
        return self.head

    def set_body(self, body):
        self.body = body
        return self

    def get_body(self):
        return self.body

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)




from paytmpg.pg.utils.stringUtil import make_string, equals


class GoodsInfo:

    def __init__(self):
        self.merchantGoodsId = None
        self.merchantShippingId = None
        self.snapshotUrl = None
        self.description = None
        self.category = None
        self.quantity = None
        self.unit = None
        self.price = None
        self.extendInfo = None

    def set_merchant_goods_id(self, merchant_goods_id):
        self.merchantGoodsId = merchant_goods_id

    def get_merchant_goods_id(self):
        return self.merchantGoodsId

    def set_merchant_shipping_id(self, merchant_shipping_id):
        self.merchantShippingId = merchant_shipping_id

    def get_merchant_shipping_id(self):
        return self.merchantShippingId

    def set_snapshot_url(self, snapshot_url):
        self.snapshotUrl = snapshot_url

    def get_snapshot_url(self):
        return self.snapshotUrl

    def set_description(self, description):
        self.description = description

    def get_description(self):
        return self.description

    def set_category(self, category):
        self.category = category

    def get_category(self):
        return self.category

    def set_quantity(self, quantity):
        self.quantity = quantity

    def get_quantity(self):
        return self.quantity

    def set_unit(self, unit):
        self.unit = unit

    def get_unit(self):
        return self.unit

    def set_price(self, price):
        self.price = price

    def get_price(self):
        return self.price

    def set_extend_info(self, extend_info):
        self.extendInfo = extend_info

    def get_extend_info(self):
        return self.extendInfo

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)

    def __repr__(self):
        return make_string(self)



from paytmpg.pg.utils.stringUtil import make_string, equals


class SDKResponse:

    response_object = None
    json_response = None

    def __init__(self):
        self.response_object = None
        self.json_response = None

    def get_response_object(self):
        return self.response_object

    def get_json_response(self):
        return self.json_response

    def set_response_object(self, response_object):
        self.response_object = response_object
        return self

    def set_json_response(self, json_response):
        self.json_response = json_response
        return self

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)



from paytmpg.pg.constants.MerchantProperty import MerchantProperty
from paytmpg.pg.request.RefundInitiateRequestBody import RefundInitiateRequestBody
from paytmpg.pg.utils.stringUtil import make_string, equals


class RefundDetail:

    def __init__(self, refund_detail_builder):
        self.__orderId = refund_detail_builder.order_id
        self.__refId = refund_detail_builder.ref_id
        self.__txnId = refund_detail_builder.txn_id
        self.__refundAmount = refund_detail_builder.refund_amount
        self.__comments = refund_detail_builder.comments
        # [dict of user_sub_wallet_type: big_decimal]
        self.__subwalletAmount = refund_detail_builder.sub_wallet_amount
        # [dict of str:object]
        self.__extraParamsMap = refund_detail_builder.extra_params_map
        self.__readTimeout = refund_detail_builder.read_timeout

    def get_order_id(self):
        return self.__orderId

    def get_ref_id(self):
        return self.__refId

    def get_txn_id(self):
        return self.__txnId

    def get_refund_amount(self):
        return self.__refundAmount

    def get_comments(self):
        return self.__comments

    def get_sub_wallet_amount(self):
        return self.__subwalletAmount

    def get_extra_params_map(self):
        return self.__extraParamsMap

    def get_read_timeout(self):
        return self.__readTimeout

    def __str__(self):
        return ""

    def __eq__(self, other):
        return equals(self)

    def __dir__(self):
        return ""

    def __create_refund_initiate_request_body(self):
        refund_initiate_request_body = RefundInitiateRequestBody()
        refund_initiate_request_body.set_mid(MerchantProperty.get_mid())
        refund_initiate_request_body.set_order_id(self.get_order_id())
        refund_initiate_request_body.set_ref_id(self.get_ref_id())
        refund_initiate_request_body.set_extra_parameter_map(self.get_extra_params_map())
        refund_initiate_request_body.set_txn_id(self.get_txn_id())
        refund_initiate_request_body.set_refund_amount(self.get_refund_amount())
        refund_initiate_request_body.set_comments(self.get_comments())
        refund_initiate_request_body.set_sub_wallet_amount(self.get_sub_wallet_amount())
        return refund_initiate_request_body


class RefundDetailBuilder:
    order_id = None
    ref_id = None
    txn_id = None
    refund_amount = None
    comments = None
    # [dict of user_sub_wallet_type: big_decimal]
    sub_wallet_amount = None
    # [dict of str:object]
    extra_params_map = None
    read_timeout = 30

    def __init__(self, order_id, ref_id, txn_id, refund_amount):
        self.order_id = order_id
        self.ref_id = ref_id
        self.txn_id = txn_id
        self.refund_amount = refund_amount

    def build(self):
        return RefundDetail(self)

    def set_order_id(self, order_id):
        self.order_id = order_id
        return self

    def set_ref_id(self, ref_id):
        self.ref_id = ref_id
        return self

    def set_extra_params_map(self, extra_params_map):
        self.extra_params_map = extra_params_map
        return self

    def set_txn_id(self, txn_id):
        self.txn_id = txn_id
        return self

    def set_refund_amount(self, refund_amount):
        self.refund_amount = refund_amount
        return self

    def set_comments(self, comments):
        self.comments = comments
        return self

    def set_sub_wallet_amount(self, sub_wallet_amount):
        self.sub_wallet_amount = sub_wallet_amount
        return self

    def set_read_timeout(self, read_timeout):
        self.read_timeout = read_timeout
        return self

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self)

    def __setattr__(self, key, value):
        if not hasattr(self, key):
            raise AttributeError(" Can not be set ".format(key))
        self.__dict__[key] = value

    def __dir__(self):
        return ""



from paytmpg.pg.utils.stringUtil import make_string, equals


class ResultInfo:

    def __init__(self):
        self.isRedirect = None
        self.resultMsg = None
        self.resultCode = None
        self.resultStatus = None

    def get_result_status(self):
        return self.resultStatus

    def set_result_status(self, result_status):
        self.resultStatus = result_status

    def set_result_code(self, result_code):
        self.resultCode = result_code

    def get_result_code(self):
        return self.resultCode

    def set_result_msg(self, result_msg):
        self.resultMsg = result_msg

    def get_result_msg(self):
        return self.resultMsg

    def is_redirect(self):
        return self.isRedirect

    def set_redirect(self, is_redirect):
        self.isRedirect = is_redirect

    def __str__(self):
        return make_string(self)

    def __eq__(self, other):
        return equals(self, other)
